package Perldoc::Server::View::Function;

use strict;
use warnings;
use parent 'Catalyst::View';

=head1 NAME

Perldoc::Server::View::Function - Catalyst View

=head1 DESCRIPTION

Catalyst View.

=head1 AUTHOR

Jon Allen

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
